
/* snoscan  - methylation guide snoRNA search program
 *
 * by Todd MJ Lowe    v. 0.1       12/08/96
 *                    v. 0.9 beta  02/19/99
 *                    v. 0.9.1     04/26/17 - updates squid library (SRE)
 *                    v. 1.0       09/04/20 - integrating snoscan multimodel features with 2017 library & code fixes\
 *
 * copyright 2020 by T. Lowe
 *
 * Uses Sean Eddy's function library for biological sequence analysis
 * (Squid v1.5.11)
 *
 *  matrices.h  - matrices used for scoring snoRNA features
 */

/* These matrices were generated from a collection of Perl scripts,
 *  using feature alignments, or collections of annotated snoRNA
 *  sequences. 
 *  
 * On my list of things ToDo: combine these scripts into one single
 * program that takes a snoRNA alignment as input.  These
 * yeast-trained matrices actually work quite well for all euks i've
 * scanned, so these defaults should work fine for general use.
 *
 * (A special set of matrices were developed to scan archaeal genomes
 * for snoRNAs, but these are still experimental and aren't ready for
 * general release to the public.)
 */ 
 
/* To see how all these matrices fit together in the overall scoring
 * scheme, see the function ScoreHit() in search.c   
 *
 * Some scoring values are still embeded in functions in search.c
 * (egad! what bad style!) -- I will move those over into this header
 * file next major work I do on the code 
 */


/* Yeast C and D' Box matrices 1/25/98 
   trained on 41 known/exp confirmed snos
   (used "gen-matrix.pl" Perl script with
   hand-alignment of feature seqs
*/


#define D_BOX_CUGA_SCORE 5.58
#define D_BOX_AUGA_SCORE 2.61


#ifdef YEAST_MODEL

#define DPBOX_TRANS_SC   -0.461   /* ln(29/46) 29 of 46 sites guided by D' boxes */ 
#define NO_DBOX_TRANS_SC -0.995   /* ln(17/46) 17 of 46 sites guided by D  boxes */

#define TERM_STEM_SC    -0.769     /* ln(19/41) 19 of 41 Sc snos have stems */
#define NO_TERM_STEM_SC -0.622     /* ln(22/41) 22 of 41 Sc snos do not have stems */ 

#define MAPPED_METH_SC     -5.10   /* ln(54 mapped/55  * (1/3*54 start sites)) */ 
#define UNMAPPED_METH_SC  -12.54   /* ln(1 unmapped/55 * (1/5100 start sites)) */

float Cbox_Mat[5][CBOX_LEN] = {
  {1.145,-2.492,-2.492,1.171,-2.492,-2.492,0.973},
  {-2.003,-2.003,-2.003,-2.003,-2.003,-2.003,-2.003},
  {-2.003,1.608,-2.003,-2.003,1.661,-2.003,-0.393},
  {-2.492,-1.799,1.119,-2.492,-2.492,1.145,-1.799},
  {-3.135,-3.135,-3.135,-3.135,-3.135,-3.135,-3.135}
};

float DpBox_Mat[5][DBOX_LEN] = {
  {1.094,-2.125,-1.432,-1.026},
  {-1.635,-1.635,-0.536,1.409},
  {-1.635,1.661,-1.635,-1.635},
  {-2.125,-2.125,0.920,-1.026},
  {-3.135,-3.135,-3.135,-3.135}
};

/* This is the archaeal training data!  -- temperary only */

float CpBox_Mat[5][CBOX_LEN] = {
{1.101, -3.058, -1.671, 1.101, -3.058, -3.058 },
{-3.170, -3.170, -2.477, -3.170, -3.170, -0.867 },
{-2.493, 1.665, -1.241, -2.493, 1.673, -1.800 },
{-3.055, -3.055, 1.006, -3.055, -3.748, 1.014 },
{-3.135, -3.135, -3.135,-3.135, -3.135, -3.135}
};

 

/* rRNA Complementarity Matrix 
   01/25/98 
   generated with "parse-snos.pl" Perl script
   from all 41 known yeast snoRNAs

   {A-A,A-C,A-G,A-U,A-N}
   {C-A,... 
   where first base is rRNA
   and second from query 
   (i.e. C-A  C (rRNA) A (query) */

float Compl_Mat[4][5] = {
  {-1.898,-1.409,-1.409,1.137,-1.30},
  {-1.626,-1.137,1.615,-1.626,-1.30},
  {-1.689,1.552,-1.200,-1.401,-1.30},
  {1.088,-1.457,-1.170,-1.947,-1.30}
};


/* Score matrices for length distributions of 
   snoRNA-rRNA duplex region lengths
   -matrices generated by arbitrarily defining bins length ranges,
   then hand-calculating scores based on bin contents

   Scores in Nats ln([counts in bin]/[totalcounts in all bins])
   Re-trained on 41 snos (50 compl regions) 10/98 */

#define GUIDE_DUPLEX_LEN_MAT_SIZE 6

int GuideDuplexLenBins_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = {9,12,15,18,21,1000};

float GuideDuplexLenSc_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = 
{-3.91,-0.776,-1.02,-2.30,-2.81,-3.91};


/* Score matrices for length distributions of gaps between features
   -again, hand-calculated
   Scores are in Nats ln([counts in bin]/[totalcounts in all bins])
   Trained from 41 Sc snoRNAs 1/26/98 
*/

/* Matrix scoring distance between C Box and rRNA compl region
   for D' box containing snoRNAs 
*/

#define CBOX_COMPLDp_MAT_SIZE 6

/* bin range delimiters */
int CBxComplDp_DistIdx_Mat[CBOX_COMPLDp_MAT_SIZE] = {5,10,15,20,25,1000};
/* scores in bins */
float CBxComplDp_DistSc_Mat[CBOX_COMPLDp_MAT_SIZE] = 
{-3.30,-1.10,-1.50,-1.22,-2.19,-5.60};  

/* Matrix scoring distance between compl region and D Box 
   for D' box containing snoRNAs */

#define COMPL_DBOX_MAT_SIZE 7

/* bin range delimiters */
int ComplDBx_DistIdx_Mat[COMPL_DBOX_MAT_SIZE] = {25,35,45,55,65,90,1000};
/* scores in bins */
float ComplDBx_DistSc_Mat[COMPL_DBOX_MAT_SIZE] =
{-5.60,-1.91,-1.10,-1.69,-1.50,-2.19,-5.60};

/* Matrix scoring distance between C Box and rRNA compl region
   for non-D' box containing snoRNAs */

#define CBOX_COMPL_MAT_SIZE 6

/* bin range delimiters */
int  CBxCompl_DistIdx_Mat[CBOX_COMPL_MAT_SIZE] = {45,55,65,75,130,1000};
/* scores in bins */
float CBxCompl_DistSc_Mat[CBOX_COMPL_MAT_SIZE] = {-2.83,-1.45,-1.04,-1.04,-2.83,-5.13};   


/* Terminal stem matrix
   generated with "term-stem-matrix" Perl script

   {A-A,A-C,A-G,A-U,A-N}
   {C-A,...                */ 

float TermStem_Mat[5][5] = {
  {-1.324,-0.834,-0.834,0.884,-1.3},
  {-1.324,-0.834,1.373,-1.324,-1.3},
  {-0.908,1.016,-0.419,-0.320,-1.3},
  {0.527,-0.419,0.169,-0.908,-1.3},
  {-1.3,-1.3,-1.3,-1.3,-1.3}
};

#endif

/* Same Matrices, Parameterized for Human snoRNAs */

#ifdef HUMAN_MODEL

#define DPBOX_TRANS_SC   -0.420   /* ln(46/70) 46 of 70 sites guided by D' boxes */ 
#define NO_DBOX_TRANS_SC -1.070   /* ln(24/70) 24 of 70 sites guided by D  boxes */

#define TERM_STEM_SC    -0.435     /* ln(44/68) have stems */
#define NO_TERM_STEM_SC -1.041     /* ln(24/68) do not have stems */ 

#define MAPPED_METH_SC     -5.77   /* ln(93 mapped/107   * (1/3*93 start sites)) */ 
#define UNMAPPED_METH_SC  -10.87   /* ln(14 unmapped/107 * (1/6900 start sites)) */

float Cbox_Mat[5][CBOX_LEN] = {
{0.783, -3.109, -3.109, 1.081, -3.109, -3.109, 0.580, },
{-1.430, -2.528, -2.528, -2.528, -2.528, -2.528, -1.430, },
{-0.935, 1.645, -1.852, -1.852, 1.660, -1.852, 0.674, },
{-0.541, -2.413, 1.084, -3.106, -3.106, 1.084, -2.413, },
{-3.135,-3.135,-3.135,-3.135,-3.135,-3.135,-3.135}
};

float DpBox_Mat[5][DBOX_LEN] = {
{1.018, -3.109, -0.624, -1.500, },
{-0.919, -2.528, -1.142, 1.461, },
{-1.852, 1.645, -0.935, -1.852, },
{-3.106, -2.413, 0.786, -0.909, },
};

float CpBox_Mat[5][CBOX_LEN] = {
  {0.741, -1.030, -0.401, 0.783, -1.500, -1.317, },
  {-0.736, -1.430, -1.142, -1.142, -1.142, -2.528, },
  {-0.753, 1.426, -0.465, -0.348, 1.533, -0.935, },
  {-0.708, -1.314, 0.655, -1.026, -2.413, 0.955, },
};

/* rRNA Complementarity Matrix 
   01/25/98 
   generated with "parse-snos.pl" Perl script
   from all 41 known yeast snoRNAs

   {A-A,A-C,A-G,A-U,A-N}
   {C-A,... 
   where first base is rRNA
   and second from query 
   (i.e. C-A  C (rRNA) A (query) */

float Compl_Mat[4][5] = {
  {-1.898,-1.409,-1.409,1.137,-1.30},
  {-1.626,-1.137,1.615,-1.626,-1.30},
  {-1.689,1.552,-1.200,-1.401,-1.30},
  {1.088,-1.457,-1.170,-1.947,-1.30}
};


/* Score matrices for length distributions of 
   snoRNA-rRNA duplex region lengths
   -matrices generated by arbitrarily defining bins length ranges,
   then hand-calculating scores based on bin contents

   Scores in Nats ln([counts in bin]/[totalcounts in all bins])
   Re-trained on Human snos (70 compl regions) 10/2000 */

#define GUIDE_DUPLEX_LEN_MAT_SIZE 8

int GuideDuplexLenBins_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = {9,10,11,12,14,15,17,1000};

float GuideDuplexLenSc_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = 
{-4.95,-1.43,-1.55,-1.37,-1.86,-2.88,-4.95,-5.65};


/* Score matrices for length distributions of gaps between features
   -again, hand-calculated
   Scores are in Nats ln([counts in bin]/[totalcounts in all bins])
   Trained from 41 Sc snoRNAs 1/26/98 
*/

/* Matrix scoring distance between C Box and rRNA compl region
   for D' box containing snoRNAs 
*/

#define CBOX_COMPLDp_MAT_SIZE 5

/* bin range delimiters */
int CBxComplDp_DistIdx_Mat[CBOX_COMPLDp_MAT_SIZE] = {1,8,12,18,1000};
/* scores in bins */
float CBxComplDp_DistSc_Mat[CBOX_COMPLDp_MAT_SIZE] = 
{-4.54,-2.30,-2.75,-5.64,-6.15};  

/* Matrix scoring distance between compl region and D Box 
   for D' box containing snoRNAs */

#define COMPL_DBOX_MAT_SIZE 11

/* bin range delimiters */
int ComplDBx_DistIdx_Mat[COMPL_DBOX_MAT_SIZE] = {15,18,21,23,25,27,32,37,40,43,1000};
/* scores in bins */
float ComplDBx_DistSc_Mat[COMPL_DBOX_MAT_SIZE] =
{-6.25,-5.06,-3.95,-2.56,-1.94,-2.45,-3.61,-3.95,-4.36,-5.05,-6.25};

/* Matrix scoring distance between C Box and rRNA compl region
   for non-D' box containing snoRNAs */

#define CBOX_COMPL_MAT_SIZE 12

/* bin range delimiters */
int  CBxCompl_DistIdx_Mat[CBOX_COMPL_MAT_SIZE] = {20,25,30,35,40,45,50,55,60,65,70,1000};
/* scores in bins */
float CBxCompl_DistSc_Mat[CBOX_COMPL_MAT_SIZE] = {-5.63,-3.33,-2.23,-1.54,-1.38,-1.94,-2.64,-3.33,-2.64,-3.33,-3.33,-5.63};   


/* Terminal stem matrix
   generated with "term-stem-matrix" Perl script

   {A-A,A-C,A-G,A-U,A-N}
   {C-A,...                */ 

float TermStem_Mat[5][5] = {
  {-1.324,-0.834,-0.834,0.884,-1.3},
  {-1.324,-0.834,1.373,-1.324,-1.3},
  {-0.908,1.016,-0.419,-0.320,-1.3},
  {0.527,-0.419,0.169,-0.908,-1.3},
  {-1.3,-1.3,-1.3,-1.3,-1.3}
};

#endif


/* Same Matrices, Parameterized for Archaeal
 snoRNAs */

#ifdef ARCH_MODEL

#define DPBOX_TRANS_SC   -0.693   /* half  sites guided by D' boxes */ 
#define NO_DBOX_TRANS_SC -0.693   /* half   sites guided by D  boxes */

#define TERM_STEM_SC     -0.223    /* ln(4/5)  4 of 5 snos have stems */
#define NO_TERM_STEM_SC  -1.61     /* ln(1/5)  1 of 5 do not have stems */ 

#define MAPPED_METH_SC    -5.30    /* ln(20 mapped/67  * (1/3*20 start sites)) */   
#define UNMAPPED_METH_SC  -8.77    /* ln(47 unmapped/67 * (1/4530 start sites)) */


float Cbox_Mat[5][CBOX_LEN] = {
{1.053, -3.751, -1.353, 1.109, -3.751, -3.751, 0.994 },
{-2.477, -3.170, -1.784, -3.170, -3.170, -3.170, -3.170 },
{-2.088, 1.665, -0.702, -3.187, 1.681, -2.088, -0.548 },
{-1.956, -2.649, 0.916, -3.055, -3.748, 1.104, -2.649 },
{-3.135, -3.135, -3.135,-3.135, -3.135, -3.135,-3.135 }
};

float DpBox_Mat[5][DBOX_LEN] = {
{1.117, -3.751, -2.365, -1.805 },
{-3.170, -3.170, -0.337, 1.601 },
{-3.187, 1.681, -1.577, -1.800 },
{-3.748, -3.748, 0.925, -2.361 },
{-3.135, -3.135, -3.135,-3.135}
};

float CpBox_Mat[5][CBOX_LEN] = {
{1.101, -3.058, -1.671, 1.101, -3.058, -3.058 },
{-3.170, -3.170, -2.477, -3.170, -3.170, -0.867 },
{-2.493, 1.665, -1.241, -2.493, 1.673, -1.800 },
{-3.055, -3.055, 1.006, -3.055, -3.748, 1.014 },
{-3.135, -3.135, -3.135,-3.135, -3.135, -3.135}
};


float DBox_Mat[5][DBOX_LEN] = {
{1.117, -3.751, -2.365, -2.141 },
{-3.170, -3.170, -0.772, 1.666 },
{-3.187, 1.681, -3.187, -3.187 },
{-3.748, -3.748, 1.014, -3.748 },
{-3.135, -3.135, -3.135,-3.135 }
};

float Compl_Mat[4][5] = {
  {-1.898,-1.409,-1.409,1.137,-1.30},
  {-1.626,-1.137,1.615,-1.626,-1.30},
  {-1.689,1.552,-1.200,-1.401,-1.30},
  {1.088,-1.457,-1.170,-1.947,-1.30}
};

#define GUIDE_DUPLEX_LEN_MAT_SIZE 6

int GuideDuplexLenBins_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = {9,10,11,12,13,1000};

float  GuideDuplexLenSc_Mat [GUIDE_DUPLEX_LEN_MAT_SIZE] = 
{-3.58,-0.944,-1.64,-1.79,-1.79,-4.84};
  

#define CBOX_COMPLDp_MAT_SIZE 6

/* bin range delimiters */
int CBxComplDp_DistIdx_Mat[CBOX_COMPLDp_MAT_SIZE] = {1,2,4,14,24,1000};
/* scores in bins */
float CBxComplDp_DistSc_Mat[CBOX_COMPLDp_MAT_SIZE] = 
{-0.87,-1.57,-2.26,-4.38,-5.70,-8.70};  

#define COMPL_DBOX_MAT_SIZE 5

/* bin range delimiters */
int ComplDBx_DistIdx_Mat[COMPL_DBOX_MAT_SIZE] = {20,25,30,35,1000};
/* scores in bins */
float ComplDBx_DistSc_Mat[COMPL_DBOX_MAT_SIZE] =
{-3.36,-1.97,-4.05,-4.97,-8.66};

#define CBOX_COMPL_MAT_SIZE 6

/* bin range delimiters */
int  CBxCompl_DistIdx_Mat[CBOX_COMPL_MAT_SIZE] = {24,30,40,55,70,1000};
/* scores in bins */
float CBxCompl_DistSc_Mat[CBOX_COMPL_MAT_SIZE] = {-3.81,-2.07,-4.90,-5.31,-5.31,-8.31};   

float TermStem_Mat[5][5] = {
  {-1.324,-0.834,-0.834,0.884,-1.3},
  {-1.324,-0.834,1.373,-1.324,-1.3},
  {-0.908,1.016,-0.419,-0.320,-1.3},
  {0.527,-0.419,0.169,-0.908,-1.3},
  {-1.3,-1.3,-1.3,-1.3,-1.3}
};

#endif

